additivemixed.smoothing<-function(n.iter = 50000, n.burnin = 10000, n.thin = 100,num.knots=3)

{#BEGIN MAIN FUNCTION
##################################################################################################
##################################################################################################
#
#CALL: 
#
#sim.par=additivemixed.smoothing(n.iter = 50000, n.burnin = 10000, n.thin = 100,num.knots=3)
#
##################################################################################################
##################################################################################################
##################################################################################################
##################################################################################################
#
#TECHNICAL DETAILS: 
#
#
#Smoothing is based on MCMC simulation implemented in WinBUGS. We use a radial basis instead of 
#the truncated polynomial basis for improved mixing 
#
#This function performs smoothing with additional random effects for the intercepts
#
#If there are additional covariates or the number of knots is different from 3 the WinBUGS file 
#needs to be changed accordingly
#
#
##################################################################################################
##################################################################################################
# 
#Requires the package R2WinBUGS (Andrew Gelman)
#
##################################################################################################
##################################################################################################
#
#
#
##################################################################################################
##################################################################################################
# 
#INPUT: number of iterations n.iter, number of burn in samples n.burnin,
#       thinning distance n.thin, and number of knots used num.knots
#	  
#
#
##################################################################################################
##################################################################################################
##################################################################################################
##################################################################################################
# 
#OUPUT: 
#
#	Returns (niter-n.burnin)/n.thin simulations from the posterior distribution of the 
#		model parameters
#
#
##################################################################################################
##################################################################################################

library(R2WinBUGS)

#Define data and program files
data.file.name="sitka.txt"
program.file.name="sit.txt"

#Read and attach data
data<-read.table(file=data.file.name,header=TRUE)
attach(data)

#Define initial values and parameters to be monitored
M<-max(id.num)
inits.U=rep(0,M)	 
inits<-function(){list(beta=c(0,0),b=c(0,0,0),taub=0.01,tauU=0.01,
                  U=inits.U,tauepsilon=0.01)}
parameters<-list("alpha","beta","b","sigmaepsilon","sigmab","sigmaU")


#Obtain the number of observations
n<-length(days)

#Define the matrix of fixed effects. This matrix could be more complex
#to incorporate other covariates. In this case the WinBUGS program should also be changed
X<-cbind(rep(1,n),days)

#Define knots used for the p-spline at equal spaced quantiles of the covariate
knots<-quantile(unique(days), seq(0,1,length=(num.knots+2))[-c(1,(num.knots+2))])

#Obtain the design matrix for random coefficients using a radial basis
Z_K<-(abs(outer(days,knots,"-")))^3
OMEGA_all<-(abs(outer(knots,knots,"-")))^3
svd.OMEGA_all<-svd(OMEGA_all)
sqrt.OMEGA_all<-t(svd.OMEGA_all$v %*% (t(svd.OMEGA_all$u)*sqrt(svd.OMEGA_all$d)))
Z<-t(solve(sqrt.OMEGA_all,t(Z_K)))

data<-list("log.size","X","Z","n","num.knots","ozone","M","id.num")



##################################################################################################
##################################################################################################
#
#
#Define initial values, parameters to be monitored and run WinBUGS
#
##################################################################################################
##################################################################################################


	Bayes.fit<- bugs(data, inits, parameters, model.file = program.file.name,
         	n.chains = 1, n.iter = n.iter, n.burnin = n.burnin,
         	n.thin = n.thin,debug = FALSE, DIC = FALSE, digits = 5, codaPkg = FALSE,
        	bugs.directory = "c:/Program Files/WinBUGS14/")

	attach.bugs(Bayes.fit)
	


#Obtain the posterior simulation from the overall mean function
m=X %*% t(beta) +Z %*% t(b)


#Obtain the median and 95% credible interval for the overall mean function
Bayes.mean.inference<-rep(0,3*nrow(m))
dim(Bayes.mean.inference)<-c(nrow(m),3)

for (i in 1:nrow(m))

	{
	Bayes.mean.inference[i,]<-quantile(m[i,],probs=c(0.025,0.5,0.975))
	} 


plot(days[1:13],Bayes.mean.inference[1:13,2],"l",col="red",lwd=2,xlab="days",ylab="Mean function",
ylim=c(min(Bayes.mean.inference[,1]),max(Bayes.mean.inference[,3])))
lines(days[1:13],Bayes.mean.inference[1:13,3],col="blue",lwd=2)
lines(days[1:13],Bayes.mean.inference[1:13,1],col="blue",lwd=2)


return(parameters)

}#END MAIN FUNCTION