semiparametric.smoothing<-function(outcome.type=1,num.knots=20,n.iter = 50000, n.burnin = 10000,
         n.thin = 100)

{#BEGIN MAIN FUNCTION
##################################################################################################
##################################################################################################
#
#CALL: 
#
#sim.par=semiparametric.smoothing(outcome.type=1,num.knots=20,n.iter = 50000, n.burnin = 10000,n.thin = 100)
#
##################################################################################################
##################################################################################################
##################################################################################################
##################################################################################################
#
#TECHNICAL DETAILS: 
#
#
#This function perfoms smoothing for generalized models (Normal, Bernoulli, Poisson). It can 
#be easily extended to accomodate other types of outcomes and additional covariates. 
#
#Smoothing is based on MCMC simulation implemented in WinBUGS. We use a radial basis instead of 
#the truncated polynomial basis for improved mixing 
#
#
#If there are additional covariates or the number of knots is different from 20 the WinBUGS file 
#needs to be changed accordingly
#
#
##################################################################################################
##################################################################################################
# 
#Requires the package R2WinBUGS (Andrew Gelman)
#
##################################################################################################
##################################################################################################
#
#
#
##################################################################################################
##################################################################################################
# 
#INPUT: 
#	1. outcome.type: 1 (normal), 2 (bernoulli), 3 (binomial), 4 (poisson)
#	2. num.knots   : number of knots for spline
#	3. file containing the outcome and the covariate  
#
#
##################################################################################################
##################################################################################################
##################################################################################################
##################################################################################################
# 
#OUPUT: 
#
#	Returns (niter-n.burnin)/n.thin simulations from the posterior distribution of the 
#		model parameters
#
#
##################################################################################################
##################################################################################################

library(R2WinBUGS)

#Read and attach data

inits.b=rep(0,20)

if (outcome.type==1)
	{data.file.name="smoothing.norm.txt"
	 program.file.name="scatter.txt"
	 inits<-function(){list(beta=c(0,0),b=inits.b,taub=0.01,taueps=0.01)}
	 parameters<-list("lambda","taub","sigmab","taueps","sigmaeps","beta","b","ystar")}
if (outcome.type==2)
	{data.file.name="smoothing.bern.txt"
	 program.file.name="bernoull.txt"
	 inits<-function(){list(beta=c(0,0),b=inits.b,taub=0.01)}
	 parameters<-list("taub","sigmab","beta","b")}
if (outcome.type==3)
	{data.file.name="smoothing.bern.txt"
	 program.file.name="poisson.txt"
	 inits<-function(){list(beta=c(0,0),b=inits.b,taub=0.01)}
	 parameters<-list("taub","sigmab","beta","b")}

data<-read.table(file=data.file.name,header=TRUE)
attach(data)

#Obtain the number of observations
n<-length(covariate)

#Define the matrix of fixed effects. This matrix could be more complex
#to incorporate other covariates. In this case the WinBUGS program should also be changed
X<-cbind(rep(1,n),covariate)

#Define knots used for the p-spline at equal spaced quantiles of the covariate
knots<-quantile(unique(covariate), seq(0,1,length=(num.knots+2))[-c(1,(num.knots+2))])

#Obtain the design matrix for random coefficients using a radial basis
Z_K<-(abs(outer(covariate,knots,"-")))^3
OMEGA_all<-(abs(outer(knots,knots,"-")))^3
svd.OMEGA_all<-svd(OMEGA_all)
sqrt.OMEGA_all<-t(svd.OMEGA_all$v %*% (t(svd.OMEGA_all$u)*sqrt(svd.OMEGA_all$d)))
Z<-t(solve(sqrt.OMEGA_all,t(Z_K)))

data<-list("response","X","Z","n","num.knots")



##################################################################################################
##################################################################################################
#
#
#Define initial values, parameters to be monitored and run WinBUGS
#
##################################################################################################
##################################################################################################


	Bayes.fit<- bugs(data, inits, parameters, model.file = program.file.name,
         	n.chains = 1, n.iter = n.iter, n.burnin = n.burnin,
         	n.thin = n.thin,debug = FALSE, DIC = FALSE, digits = 5, codaPkg = FALSE,
        	bugs.directory = "c:/Program Files/WinBUGS14/")

	attach.bugs(Bayes.fit)
	


#Obtain the posterior simulation from the overall mean function
m=X %*% t(beta) +Z %*% t(b)


#Obtain the median and 95% credible interval for the overall mean function
Bayes.mean.inference<-rep(0,3*nrow(m))
dim(Bayes.mean.inference)<-c(nrow(m),3)

for (i in 1:nrow(m))

	{
	Bayes.mean.inference[i,]<-quantile(m[i,],probs=c(0.025,0.5,0.975))
	} 

#Transform back to the outcome scale
if (outcome.type==2){Bayes.mean.inference<-exp(Bayes.mean.inference)/(1+exp(Bayes.mean.inference))}
if (outcome.type==3){Bayes.mean.inference<-exp(Bayes.mean.inference)}

#Re-order variables for plotting purposes
o=order(covariate)
covariates=covariate[o]
ordered.mean.Bayes=Bayes.mean.inference[o,]

plot(covariates,ordered.mean.Bayes[,2],"l",col="red",lwd=2,xlab="Covariate",ylab="Mean function",
ylim=c(min(ordered.mean.Bayes[,1]),max(ordered.mean.Bayes[,3])))
lines(covariates,ordered.mean.Bayes[,3],col="blue",lwd=2)
lines(covariates,ordered.mean.Bayes[,1],col="blue",lwd=2)


return(parameters)

}#END MAIN FUNCTION